package Sophos::GD::Gradient;

use 5.006;
use strict;
use warnings;
use constant pi     =>  4 * atan2 1, 1;
use constant torad  =>  pi() / 180;

require Exporter;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(apply_gradient);
our $VERSION = '0.001';

return 1;

sub apply_gradient {
    my($image, $colour, $gradient, $angle, $opacity) = @_;
    my($width, $height) = $image->getBounds();
    return unless($width && $height);
    my @match;
    my $colours = @$gradient;
    $opacity = 1 unless defined $opacity;

    if(ref $colour) {
        @match = @$colour;
    } else {
        @match = ($colour);
    }
    
    # how many pixels thick each gradient colour should be
    my $ratio = $colours / $height;
    
    # an angle of "0" would be a straight gradient from bottom-to-top
    # (0 = north)

    my $radians = $angle * torad;
    my($adjx, $adjy) = (cos($radians), sin($radians));
    
    for(my $x = 0; $x < $width; $x++) {
        for(my $y = 0; $y < $height; $y++) {
            my $incolour = $image->getPixel($x, $y);
            if((!@match) || (grep($incolour == $_, @match))) {
                my(@rcol) = $image->rgb($incolour);
                # rotate to our angle
                my $rx = ($x * $adjx) + ($y * $adjy);
                my $ry = ($y * $adjx) + ($x * $adjy);
                
                my $idx = _overlap($colours, int($ry * $ratio));
                my @gcol = map { int $_ } @{$gradient->[$idx]};
                my $opa;
                
                if(defined $gcol[3]) {
                    $opa = $opacity * ($gcol[3] / 255);
                } else {
                    $opa = $opacity;
                }
                
                foreach my $i (0..2) {
                    $gcol[$i] = ($gcol[$i] * $opa) +
                        ($rcol[$i] * (1-$opa));
                }
                
                my $icol;
                
                if(($icol = $image->colorExact(@gcol[0..2])) == -1) {
                    $icol = $image->colorAllocate(@gcol[0..2]);
                }
                
                $image->setPixel($x, $y, $icol);
            }
        }
    }
    
    return $image;
}

# this could be faster.
sub _overlap {
    my($limit, $value) = @_;
    
    while($value < 0) {
        $value += $limit;
    }
    
    while($value >= $limit) {
        $value -= $limit;
    }
    
    return $value;
}

# Preloaded methods go here.

__END__
